@@ -2,8 +2,10 @@
 from __future__ import unicode_literals
 
 import re
+import time
 import logging
 from datetime import datetime
+from timeit import default_timer as timer
 
 import six
 from kitchen.text.display import wrap
@@ -11,6 +13,8 @@ from kitchen.text.display import wrap
 from . import exceptions
 from .packages import praw
 from .packages.praw.errors import InvalidSubreddit
+from .packages.praw.helpers import normalize_url
+from .packages.praw.handlers import DefaultHandler
 
 _logger = logging.getLogger(__name__)
 
@@ -213,6 +217,7 @@ class Content(object):
         data['title'] = sub.title
         data['text'] = sub.selftext
         data['created'] = cls.humanize_timestamp(sub.created_utc)
+        data['created_long'] = cls.humanize_timestamp(sub.created_utc, True)
         data['comments'] = '{0} comments'.format(sub.num_comments)
         data['score'] = '{0} pts'.format('-' if sub.hide_score else sub.score)
         data['author'] = name
@@ -437,12 +442,13 @@ class SubredditContent(Content):
     """
 
     def __init__(self, name, submissions, loader, order=None,
-                 max_title_rows=4, query=None):
+                 max_title_rows=4, query=None, filter_nsfw=False):
 
         self.name = name
         self.order = order
         self.query = query
         self.max_title_rows = max_title_rows
+        self.filter_nsfw = filter_nsfw
         self._loader = loader
         self._submissions = submissions
         self._submission_data = []
@@ -477,22 +483,33 @@ class SubredditContent(Content):
             query (text): Content to search for on the given subreddit or
                 user's page.
         """
-        # TODO: refactor this into smaller methods
+        # TODO: This desperately needs to be refactored
 
         # Strip leading, trailing, and redundant backslashes
         parts = [seg for seg in name.strip(' /').split('/') if seg]
 
         # Check for the resource type, assume /r/ as the default
         if len(parts) >= 3 and parts[2] == 'm':
-            # E.g. /u/multi-mod/m/android
+            # E.g. /u/civilization_phaze_3/m/multireddit ->
+            #    resource_root = "u/civilization_phaze_3/m"
+            #    parts = ["multireddit"]
             resource_root, parts = '/'.join(parts[:3]), parts[3:]
         elif len(parts) > 1 and parts[0] in ['r', 'u', 'user', 'domain']:
+            # E.g. /u/civilization_phaze_3 ->
+            #    resource_root = "u"
+            #    parts = ["civilization_phaze_3"]
+            #
+            # E.g. /r/python/top-week ->
+            #    resource_root = "r"
+            #    parts = ["python", "top-week"]
             resource_root = parts.pop(0)
         else:
             resource_root = 'r'
 
         if resource_root == 'user':
             resource_root = 'u'
+        elif resource_root.startswith('user/'):
+            resource_root = 'u' + resource_root[4:]
 
         # There should at most two parts left, the resource and the order
         if len(parts) == 1:
@@ -520,13 +537,21 @@ class SubredditContent(Content):
         else:
             period = None
 
-        if order not in ['hot', 'top', 'rising', 'new', 'controversial', None]:
+        if query:
+            # The allowed orders for sorting search results are different
+            orders = ['relevance', 'top', 'comments', 'new', None]
+            period_allowed = ['top', 'comments']
+        else:
+            orders = ['hot', 'top', 'rising', 'new', 'controversial', None]
+            period_allowed = ['top', 'controversial']
+
+        if order not in orders:
             raise InvalidSubreddit('Invalid order `%s`' % order)
         if period not in ['all', 'day', 'hour', 'month', 'week', 'year', None]:
             raise InvalidSubreddit('Invalid period `%s`' % period)
-        if period and order not in ['top', 'controversial']:
-            raise InvalidSubreddit('`%s` order does not allow sorting by'
-                                   ' period' % order)
+        if period and order not in period_allowed:
+            raise InvalidSubreddit(
+                '`%s` order does not allow sorting by period' % order)
 
         # On some objects, praw doesn't allow you to pass arguments for the
         # order and period. Instead you need to call special helper functions
@@ -561,6 +586,15 @@ class SubredditContent(Content):
 
         elif resource_root.endswith('/m'):
             redditor = resource_root.split('/')[1]
+
+            if redditor == 'me':
+                if not reddit.is_oauth_session():
+                    raise exceptions.AccountError('Not logged in')
+                else:
+                    redditor = reddit.user.name
+                    display_name = display_name.replace(
+                        '/me/', '/{0}/'.format(redditor))
+
             multireddit = reddit.get_multireddit(redditor, resource)
             submissions = getattr(multireddit, method_alias)(limit=None)
 
@@ -605,9 +639,11 @@ class SubredditContent(Content):
             # display name with the one returned by the request.
             display_name = '/r/{0}'.format(subreddit.display_name)
 
+        filter_nsfw = (reddit.user and reddit.user.over_18 is False)
+
         # We made it!
         return cls(display_name, submissions, loader, order=display_order,
-                   query=query)
+                   query=query, filter_nsfw=filter_nsfw)
 
     @property
     def range(self):
@@ -625,6 +661,7 @@ class SubredditContent(Content):
         if index < 0:
             raise IndexError
 
+        nsfw_count = 0
         while index >= len(self._submission_data):
             try:
                 with self._loader('Loading more submissions'):
@@ -634,10 +671,25 @@ class SubredditContent(Content):
             except StopIteration:
                 raise IndexError
             else:
+
+                # Skip NSFW posts based on the reddit user's profile settings.
+                # If we see 20+ NSFW posts at the beginning, assume the subreddit
+                # only has NSFW content and abort. This allows us to avoid making
+                # an additional API call to check if a subreddit is over18 (which
+                # doesn't work for things like multireddits anyway)
+                if self.filter_nsfw and submission.over_18:
+                    nsfw_count += 1
+                    if not self._submission_data and nsfw_count >= 20:
+                        raise exceptions.SubredditError(
+                            'You must be over 18+ to view this subreddit')
+                    continue
+                else:
+                    nsfw_count = 0
+
                 if hasattr(submission, 'title'):
                     data = self.strip_praw_submission(submission)
                 else:
-                    # when submission is a saved commment
+                    # when submission is a saved comment
                     data = self.strip_praw_comment(submission)
 
                 data['index'] = len(self._submission_data) + 1
@@ -674,7 +726,7 @@ class SubscriptionContent(Content):
             raise exceptions.SubscriptionError('No content')
 
         # Load 1024 subscriptions up front (one http request's worth)
-        # For most people this should be all of their subscriptions. Doing thi
+        # For most people this should be all of their subscriptions. This
         # allows the user to jump to the end of the page with `G`.
         if name != 'Popular Subreddits':
             try:
@@ -730,3 +782,169 @@ class SubscriptionContent(Content):
         data['h_offset'] = 0
 
         return data
+
+
+class RequestHeaderRateLimiter(DefaultHandler):
+    """Custom PRAW request handler for rate-limiting requests.
+
+    This is an alternative to PRAW 3's DefaultHandler that uses
+    Reddit's modern API guidelines to rate-limit requests based
+    on the X-Ratelimit-* headers returned from Reddit. Most of
+    these methods are copied from or derived from the DefaultHandler.
+
+    References:
+        https://github.com/reddit/reddit/wiki/API
+        https://github.com/praw-dev/prawcore/blob/master/prawcore/rate_limit.py
+    """
+
+    def __init__(self):
+
+        # In PRAW's convention, these variables were bound to the
+        # class so the cache could be shared among all of the ``reddit``
+        # instances. In RTV's use-case there is only ever a single reddit
+        # instance so it made sense to clean up the globals and transfer them
+        # to method variables
+        self.cache = {}
+        self.timeouts = {}
+
+        # These are used for the header rate-limiting
+        self.used = None
+        self.remaining = None
+        self.seconds_to_reset = None
+        self.next_request_timestamp = None
+
+        super(RequestHeaderRateLimiter, self).__init__()
+
+    def _delay(self):
+        """
+        Pause before making the next HTTP request.
+        """
+        if self.next_request_timestamp is None:
+            return
+
+        sleep_seconds = self.next_request_timestamp - time.time()
+        if sleep_seconds <= 0:
+            return
+        time.sleep(sleep_seconds)
+
+    def _update(self, response_headers):
+        """
+        Update the state of the rate limiter based on the response headers:
+
+            X-Ratelimit-Used: Approximate number of requests used this period
+            X-Ratelimit-Remaining: Approximate number of requests left to use
+            X-Ratelimit-Reset: Approximate number of seconds to end of period
+
+        PRAW 5's rate limiting logic is structured for making hundreds of
+        evenly-spaced API requests, which makes sense for running something
+        like a bot or crawler.
+
+        This handler's logic, on the other hand, is geared more towards
+        interactive usage. It allows for short, sporadic bursts of requests.
+        The assumption is that actual users browsing reddit shouldn't ever be
+        in danger of hitting the rate limit. If they do hit the limit, they
+        will be cutoff until the period resets.
+        """
+
+        if 'x-ratelimit-remaining' not in response_headers:
+            # This could be because the API returned an error response, or it
+            # could be because we're using something like read-only credentials
+            # which Reddit doesn't appear to care about rate limiting.
+            return
+
+        self.used = float(response_headers['x-ratelimit-used'])
+        self.remaining = float(response_headers['x-ratelimit-remaining'])
+        self.seconds_to_reset = int(response_headers['x-ratelimit-reset'])
+        _logger.debug('Rate limit: %s used, %s remaining, %s reset',
+                      self.used, self.remaining, self.seconds_to_reset)
+
+        if self.remaining <= 0:
+            self.next_request_timestamp = time.time() + self.seconds_to_reset
+        else:
+            self.next_request_timestamp = None
+
+    def _clear_timeouts(self, cache_timeout):
+        """
+        Clear the cache of timed out results.
+        """
+
+        for key in list(self.timeouts):
+            if timer() - self.timeouts[key] > cache_timeout:
+                del self.timeouts[key]
+                del self.cache[key]
+
+    def clear_cache(self):
+        """Remove all items from the cache."""
+        self.cache = {}
+        self.timeouts = {}
+
+    def evict(self, urls):
+        """Remove items from cache matching URLs.
+
+        Return the number of items removed.
+
+        """
+        if isinstance(urls, six.text_type):
+            urls = [urls]
+        urls = set(normalize_url(url) for url in urls)
+        retval = 0
+        for key in list(self.cache):
+            if key[0] in urls:
+                retval += 1
+                del self.cache[key]
+                del self.timeouts[key]
+        return retval
+
+    def request(self, _cache_key, _cache_ignore, _cache_timeout, **kwargs):
+        """
+        This is a wrapper function that handles the caching of the request.
+        
+        See DefaultHandler.with_cache for reference.
+        """
+        if _cache_key:
+            # Pop the request's session cookies from the cache key.
+            # These appear to be unreliable and change with every
+            # request. Also, with the introduction of OAuth I don't think
+            # that cookies are being used to store anything that
+            # differentiates API requests anyways
+            url, items = _cache_key
+            _cache_key = (url, (items[0], items[1], items[3], items[4]))
+
+        if kwargs['request'].method != 'GET':
+            # I added this check for RTV, I have no idea why PRAW would ever
+            # want to cache POST/PUT/DELETE requests
+            _cache_ignore = True
+
+        if _cache_ignore:
+            return self._request(**kwargs)
+
+        self._clear_timeouts(_cache_timeout)
+        if _cache_key in self.cache:
+            return self.cache[_cache_key]
+
+        result = self._request(**kwargs)
+
+        # The handlers don't call `raise_for_status` so we need to ignore
+        # status codes that will result in an exception that should not be
+        # cached.
+        if result.status_code not in (200, 302):
+            return result
+
+        self.timeouts[_cache_key] = timer()
+        self.cache[_cache_key] = result
+        return result
+
+    def _request(self, request, proxies, timeout, verify, **_):
+        """
+        This is where we apply rate limiting and make the HTTP request.
+        """
+
+        settings = self.http.merge_environment_settings(
+            request.url, proxies, False, verify, None)
+
+        self._delay()
+        response = self.http.send(
+            request, timeout=timeout, allow_redirects=False, **settings)
+        self._update(response.headers)
+
+        return response
