@@ -7,12 +7,18 @@ desktop:https://github.com/michael-lazar/rtv:{version}\
 """
 
 SUMMARY = """
-Reddit Terminal Viewer is a lightweight browser for www.reddit.com built into a
-terminal window.
+RTV (Reddit Terminal Viewer) is a terminal interface to view and interact with reddit.
+"""
+
+USAGE = """\
+rtv [URL] [-s SUBREDDIT]
+
+  $ rtv https://www.reddit.com/r/programming/comments/7h9l31
+  $ rtv -s linux
 """
 
 CONTROLS = """
-Move the cursor using either the arrow keys or *Vim* style movement.
+Move the cursor using the arrow keys or vim style movement.
 Press `?` to open the help screen.
 """
 
@@ -61,6 +67,8 @@ https://github.com/michael-lazar/rtv
   b     : Display urls with urlview
   y     : Copy submission permalink to clipboard
   Y     : Copy submission link to clipboard
+  F2    : Cycle to previous theme
+  F3    : Cycle to next theme
 
 [Prompt]
   The `/` prompt accepts subreddits in the following formats
@@ -84,6 +92,10 @@ BANNER = """
 [1]hot [2]top [3]rising [4]new [5]controversial
 """
 
+BANNER_SEARCH = """
+[1]relevance [2]top [3]comments [4]new
+"""
+
 FOOTER_SUBREDDIT = """
 [?]Help [q]Quit [l]Comments [/]Prompt [u]Login [o]Open [c]Post [a/z]Vote
 """
