@@ -32,6 +32,7 @@ class BaseMIMEParser(object):
                 browser.
         """
         filename = url.split('?')[0]
+        filename = filename.split('#')[0]
         content_type, _ = mimetypes.guess_type(filename)
         return url, content_type
 
@@ -54,12 +55,13 @@ class OpenGraphMIMEParser(BaseMIMEParser):
     def get_mimetype(url):
         page = requests.get(url)
         soup = BeautifulSoup(page.content, 'html.parser')
-        tag = soup.find('meta', attrs={'property': 'og:video:secure_url'})
-        tag = tag or soup.find('meta', attrs={'property':  'og:image'})
-        if tag:
-            return BaseMIMEParser.get_mimetype(tag.get('content'))
-        else:
-            return url, None
+        for og_type in ['video', 'image']:
+            tag = soup.find('meta',
+                    attrs={'property':'og:' + og_type + ':secure_url'}) or \
+                  soup.find('meta', attrs={'property': 'og:' + og_type})
+            if tag:
+                return BaseMIMEParser.get_mimetype(tag.get('content'))
+        return url, None
 
 
 class GfycatMIMEParser(BaseMIMEParser):
@@ -77,7 +79,7 @@ class GfycatMIMEParser(BaseMIMEParser):
 
     @staticmethod
     def get_mimetype(url):
-        parts = url.split('/')
+        parts = url.replace('gifs/detail/', '').split('/')
         api_url = '/'.join(parts[:-1] + ['cajax', 'get'] + parts[-1:])
         resp = requests.get(api_url)
         image_url = resp.json()['gfyItem']['webmUrl']
@@ -128,21 +130,45 @@ class RedditUploadsMIMEParser(BaseMIMEParser):
         return url, content_type
 
 
+class RedditVideoMIMEParser(BaseMIMEParser):
+    """
+    Reddit hosted videos/gifs.
+    Media uses MPEG-DASH format (.mpd)
+    """
+    pattern = re.compile(r'https://v\.redd\.it/.+$')
+
+    @staticmethod
+    def get_mimetype(url):
+        request_url = url + '/DASHPlaylist.mpd'
+        page = requests.get(request_url)
+        soup = BeautifulSoup(page.content, 'html.parser')
+        if not soup.find('representation', attrs={'mimetype': 'audio/mp4'}):
+            reps = soup.find_all('representation',
+                                 attrs={'mimetype': 'video/mp4'})
+            rep = sorted(reps, reverse=True,
+                         key=lambda t: int(t.get('bandwidth')))[0]
+            return url + '/' + rep.find('baseurl').text, 'video/mp4'
+        else:
+            return request_url, 'video/x-youtube'
+
+
 class ImgurApiMIMEParser(BaseMIMEParser):
-    """   
+    """
     Imgur now provides a json API exposing its entire infrastructure. Each Imgur
     page has an associated hash and can either contain an album, a gallery,
     or single image.
-    
+
     The default client token for RTV is shared among users and allows a maximum
     global number of requests per day of 12,500. If we find that this limit is
     not sufficient for all of rtv's traffic, this method will be revisited.
-    
+
     Reference:
         https://apidocs.imgur.com
     """
     CLIENT_ID = None
-    pattern = re.compile(r'https?://(w+\.)?(m\.)?imgur\.com/((?P<domain>a|album|gallery)/)?(?P<hash>[a-zA-Z0-9]+)$')
+    pattern = re.compile(
+        r'https?://(w+\.)?(m\.)?imgur\.com/'
+        r'((?P<domain>a|album|gallery)/)?(?P<hash>[a-zA-Z0-9]+)$')
 
     @classmethod
     def get_mimetype(cls, url):
@@ -182,11 +208,14 @@ class ImgurApiMIMEParser(BaseMIMEParser):
             _logger.warning('Imgur API failure, resp %s', r.json())
             return cls.fallback(url, domain)
 
-        if 'images' in data:
+        if 'images' in data and len(data['images']) > 1:
             # TODO: handle imgur albums with mixed content, i.e. jpeg and gifv
             link = ' '.join([d['link'] for d in data['images'] if not d['animated']])
             mime = 'image/x-imgur-album'
         else:
+            data = data['images'][0] if 'images' in data else data
+            # this handles single image galleries
+
             link = data['mp4'] if data['animated'] else data['link']
             mime = 'video/mp4' if data['animated'] else data['type']
 
@@ -280,6 +309,32 @@ class StreamableMIMEParser(OpenGraphMIMEParser):
     pattern = re.compile(r'https?://(www\.)?streamable\.com/[^.]+$')
 
 
+class TwitchMIMEParser(BaseMIMEParser):
+    """
+    Non-streaming videos hosted by twitch.tv
+    """
+    pattern = re.compile(r'https?://clips\.?twitch\.tv/[^.]+$')
+
+    @staticmethod
+    def get_mimetype(url):
+        page = requests.get(url)
+        soup = BeautifulSoup(page.content, 'html.parser')
+        tag = soup.find('meta', attrs={'name': 'twitter:image'})
+        thumbnail = tag.get('content')
+        suffix = '-preview.jpg'
+        if thumbnail.endswith(suffix):
+            return thumbnail.replace(suffix, '.mp4'), 'video/mp4'
+        else:
+            return url, None
+
+
+class OddshotMIMEParser(OpenGraphMIMEParser):
+    """
+    Oddshot uses the Open Graph protocol
+    """
+    pattern = re.compile(r'https?://oddshot\.tv/s(hot)?/[^.]+$')
+
+
 class VidmeMIMEParser(BaseMIMEParser):
     """
     Vidme provides a json api.
@@ -321,21 +376,131 @@ class LiveleakMIMEParser(BaseMIMEParser):
             if source:
                 urls.append((source.get('src'), source.get('type')))
         # TODO: Handle pages with multiple videos
-        # TODO: Handle pages with youtube embeds
         if urls:
             return urls[0]
         else:
-            return url, None
+            iframe = soup.find_all(lambda t: t.name == 'iframe' and
+                                    'youtube.com' in t['src'])
+            if iframe:
+                return YoutubeMIMEParser.get_mimetype(iframe[0]['src'].strip('/'))
+            else:
+                return url, None
+
+
+class ClippitUserMIMEParser(BaseMIMEParser):
+    """
+    """
+    pattern = re.compile(r'https?://(www\.)?clippituser\.tv/c/.+$')
+
+    @staticmethod
+    def get_mimetype(url):
+        page = requests.get(url)
+        soup = BeautifulSoup(page.content, 'html.parser')
+        tag = soup.find(id='jwplayer-container')
+        quality = ['data-{}-file'.format(_) for _ in ['hd', 'sd']]
+        return tag.get(quality[0]), 'video/mp4'
+
+
+class GifsMIMEParser(OpenGraphMIMEParser):
+    """
+    Gifs.com uses the Open Graph protocol
+    """
+    pattern = re.compile(r'https?://(www\.)?gifs\.com/gif/.+$')
+
+
+class GiphyMIMEParser(OpenGraphMIMEParser):
+    """
+    Giphy.com uses the Open Graph protocol
+    """
+    pattern = re.compile(r'https?://(www\.)?giphy\.com/gifs/.+$')
+
+
+class ImgtcMIMEParser(OpenGraphMIMEParser):
+    """
+    imgtc.com uses the Open Graph protocol
+    """
+    pattern = re.compile(r'https?://(www\.)?imgtc\.com/w/.+$')
+
+
+class ImgflipMIMEParser(OpenGraphMIMEParser):
+    """
+    imgflip.com uses the Open Graph protocol
+    """
+    pattern = re.compile(r'https?://(www\.)?imgflip\.com/i/.+$')
+
+
+class LivememeMIMEParser(OpenGraphMIMEParser):
+    """
+    livememe.com uses the Open Graph protocol
+    """
+    pattern = re.compile(r'https?://(www\.)?livememe\.com/[^.]+$')
+
+
+class MakeamemeMIMEParser(OpenGraphMIMEParser):
+    """
+    makeameme.com uses the Open Graph protocol
+    """
+    pattern = re.compile(r'https?://(www\.)?makeameme\.org/meme/.+$')
+
+
+class FlickrMIMEParser(OpenGraphMIMEParser):
+    """
+    Flickr uses the Open Graph protocol
+    """
+    pattern = re.compile(r'https?://(www\.)?flickr\.com/photos/[^/]+/[^/]+/?$')
+    # TODO: handle albums/photosets (https://www.flickr.com/services/api)
+
+
+class WorldStarHipHopMIMEParser(BaseMIMEParser):
+    """
+    <video>
+        <source src="https://hw-mobile.worldstarhiphop.com/..mp4" type="video/mp4">
+        <source src="" type="video/mp4">
+    </video>
+    Sometimes only one video source is available
+    """
+    pattern = re.compile(r'https?://((www|m)\.)?worldstarhiphop\.com/videos/video.php\?v=\w+$')
+
+    @staticmethod
+    def get_mimetype(url):
+        page = requests.get(url)
+        soup = BeautifulSoup(page.content, 'html.parser')
+
+        source = soup.find_all(lambda t: t.name == 'source' and
+                                t['src'] and t['type'] == 'video/mp4')
+        if source:
+            return source[0]['src'], 'video/mp4'
+        else:
+            iframe = soup.find_all(lambda t: t.name == 'iframe' and
+                                    'youtube.com' in t['src'])
+            if iframe:
+                return YoutubeMIMEParser.get_mimetype(iframe[0]['src'])
+            else:
+                return url, None
+
+
 
 # Parsers should be listed in the order they will be checked
 parsers = [
+    ClippitUserMIMEParser,
+    OddshotMIMEParser,
     StreamableMIMEParser,
     VidmeMIMEParser,
     InstagramMIMEParser,
     GfycatMIMEParser,
     ImgurApiMIMEParser,
     RedditUploadsMIMEParser,
+    RedditVideoMIMEParser,
     YoutubeMIMEParser,
     LiveleakMIMEParser,
+    TwitchMIMEParser,
+    FlickrMIMEParser,
+    GifsMIMEParser,
+    GiphyMIMEParser,
+    ImgtcMIMEParser,
+    ImgflipMIMEParser,
+    LivememeMIMEParser,
+    MakeamemeMIMEParser,
+    WorldStarHipHopMIMEParser,
     GifvMIMEParser,
     BaseMIMEParser]
