@@ -4,7 +4,6 @@ from __future__ import unicode_literals
 import os
 import sys
 import time
-import curses
 import logging
 from functools import wraps
 
@@ -12,7 +11,7 @@ import six
 from kitchen.text.display import textual_width
 
 from . import docs
-from .objects import Controller, Color, Command
+from .objects import Controller, Command
 from .clipboard import copy
 from .exceptions import TemporaryFileError, ProgramError
 from .__version__ import __version__
@@ -82,6 +81,11 @@ class Page(object):
             ch = self.term.stdscr.getch()
             self.controller.trigger(ch)
 
+    @PageController.register(Command('REFRESH'))
+    def reload_page(self):
+        self.reddit.handler.clear_cache()
+        self.refresh_content()
+
     @PageController.register(Command('EXIT'))
     def exit(self):
         if self.term.prompt_y_or_n('Do you really want to quit? (y/n): '):
@@ -91,64 +95,34 @@ class Page(object):
     def force_exit(self):
         sys.exit()
 
+    @PageController.register(Command('PREVIOUS_THEME'))
+    def previous_theme(self):
+
+        theme = self.term.theme_list.previous(self.term.theme)
+        while not self.term.check_theme(theme):
+            theme = self.term.theme_list.previous(theme)
+
+        self.term.set_theme(theme)
+        self.draw()
+        message = self.term.theme.display_string
+        self.term.show_notification(message, timeout=1)
+
+    @PageController.register(Command('NEXT_THEME'))
+    def next_theme(self):
+
+        theme = self.term.theme_list.next(self.term.theme)
+        while not self.term.check_theme(theme):
+            theme = self.term.theme_list.next(theme)
+
+        self.term.set_theme(theme)
+        self.draw()
+        message = self.term.theme.display_string
+        self.term.show_notification(message, timeout=1)
+
     @PageController.register(Command('HELP'))
     def show_help(self):
         self.term.open_pager(docs.HELP.strip())
 
-    @PageController.register(Command('SORT_HOT'))
-    def sort_content_hot(self):
-        self.refresh_content(order='hot')
-
-    @PageController.register(Command('SORT_TOP'))
-    def sort_content_top(self):
-
-        choices = {
-            '\n': 'top',
-            '1': 'top-hour',
-            '2': 'top-day',
-            '3': 'top-week',
-            '4': 'top-month',
-            '5': 'top-year',
-            '6': 'top-all'}
-
-        message = docs.TIME_ORDER_MENU.strip().splitlines()
-        ch = self.term.show_notification(message)
-        ch = six.unichr(ch)
-        if ch not in choices:
-            self.term.show_notification('Invalid option')
-            return
-
-        self.refresh_content(order=choices[ch])
-
-    @PageController.register(Command('SORT_RISING'))
-    def sort_content_rising(self):
-        self.refresh_content(order='rising')
-
-    @PageController.register(Command('SORT_NEW'))
-    def sort_content_new(self):
-        self.refresh_content(order='new')
-
-    @PageController.register(Command('SORT_CONTROVERSIAL'))
-    def sort_content_controversial(self):
-
-        choices = {
-            '\n': 'controversial',
-            '1': 'controversial-hour',
-            '2': 'controversial-day',
-            '3': 'controversial-week',
-            '4': 'controversial-month',
-            '5': 'controversial-year',
-            '6': 'controversial-all'}
-
-        message = docs.TIME_ORDER_MENU.strip().splitlines()
-        ch = self.term.show_notification(message)
-        ch = six.unichr(ch)
-        if ch not in choices:
-            self.term.show_notification('Invalid option')
-            return
-
-        self.refresh_content(order=choices[ch])
-
     @PageController.register(Command('MOVE_UP'))
     def move_cursor_up(self):
         self._move_cursor(-1)
@@ -171,19 +145,15 @@ class Page(object):
 
     @PageController.register(Command('PAGE_TOP'))
     def move_page_top(self):
-        self._remove_cursor()
         self.nav.page_index = self.content.range[0]
         self.nav.cursor_index = 0
         self.nav.inverted = False
-        self._add_cursor()
 
     @PageController.register(Command('PAGE_BOTTOM'))
     def move_page_bottom(self):
-        self._remove_cursor()
         self.nav.page_index = self.content.range[1]
         self.nav.cursor_index = 0
         self.nav.inverted = True
-        self._add_cursor()
 
     @PageController.register(Command('UPVOTE'))
     @logged_in
@@ -244,7 +214,8 @@ class Page(object):
         """
 
         if self.reddit.is_oauth_session():
-            if self.term.prompt_y_or_n('Log out? (y/n): '):
+            ch = self.term.show_notification('Log out? (y/n)')
+            if ch in (ord('y'), ord('Y')):
                 self.oauth.clear_oauth_data()
                 self.term.show_notification('Logged out')
         else:
@@ -272,7 +243,7 @@ class Page(object):
             # Give reddit time to process the request
             time.sleep(2.0)
         if self.term.loader.exception is None:
-            self.refresh_content()
+            self.reload_page()
 
     @PageController.register(Command('EDIT'))
     @logged_in
@@ -308,7 +279,7 @@ class Page(object):
                 time.sleep(2.0)
 
             if self.term.loader.exception is None:
-                self.refresh_content()
+                self.reload_page()
             else:
                 raise TemporaryFileError()
 
@@ -389,7 +360,6 @@ class Page(object):
         self._draw_banner()
         self._draw_content()
         self._draw_footer()
-        self._add_cursor()
         self.term.clear_screen()
         self.term.stdscr.refresh()
 
@@ -401,8 +371,7 @@ class Page(object):
         window = self.term.stdscr.derwin(1, n_cols, self._row, 0)
         window.erase()
         # curses.bkgd expects bytes in py2 and unicode in py3
-        ch, attr = str(' '), curses.A_REVERSE | curses.A_BOLD | Color.CYAN
-        window.bkgd(ch, attr)
+        window.bkgd(str(' '), self.term.attr('TitleBar'))
 
         sub_name = self.content.name
         sub_name = sub_name.replace('/r/front', 'Front Page')
@@ -422,7 +391,9 @@ class Page(object):
         else:
             title = sub_name
 
-        if os.getenv('DISPLAY'):
+        # Setting the terminal title will break emacs or systems without
+        # X window.
+        if os.getenv('DISPLAY') and not os.getenv('INSIDE_EMACS'):
             title += ' - rtv {0}'.format(__version__)
             title = self.term.clean(title)
             if six.PY3:
@@ -434,7 +405,7 @@ class Page(object):
             sys.stdout.write(title)
             sys.stdout.flush()
 
-        if self.reddit.user is not None:
+        if self.reddit and self.reddit.user is not None:
             # The starting position of the name depends on if we're converting
             # to ascii or not
             width = len if self.config['ascii'] else textual_width
@@ -455,10 +426,11 @@ class Page(object):
         n_rows, n_cols = self.term.stdscr.getmaxyx()
         window = self.term.stdscr.derwin(1, n_cols, self._row, 0)
         window.erase()
-        ch, attr = str(' '), curses.A_BOLD | Color.YELLOW
-        window.bkgd(ch, attr)
+        window.bkgd(str(' '), self.term.attr('OrderBar'))
+
+        banner = docs.BANNER_SEARCH if self.content.query else docs.BANNER
+        items = banner.strip().split(' ')
 
-        items = docs.BANNER.strip().split(' ')
         distance = (n_cols - sum(len(t) for t in items) - 1) / (len(items) - 1)
         spacing = max(1, int(distance)) * ' '
         text = spacing.join(items)
@@ -466,7 +438,8 @@ class Page(object):
         if self.content.order is not None:
             order = self.content.order.split('-')[0]
             col = text.find(order) - 3
-            window.chgat(0, col, 3, attr | curses.A_REVERSE)
+            attr = self.term.attr('OrderBarHighlight')
+            window.chgat(0, col, 3, attr)
 
         self._row += 1
 
@@ -476,8 +449,7 @@ class Page(object):
         """
 
         n_rows, n_cols = self.term.stdscr.getmaxyx()
-        window = self.term.stdscr.derwin(
-            n_rows - self._row - 1, n_cols, self._row, 0)
+        window = self.term.stdscr.derwin(n_rows - self._row - 1, n_cols, self._row, 0)
         window.erase()
         win_n_rows, win_n_cols = window.getmaxyx()
 
@@ -504,10 +476,8 @@ class Page(object):
                 top_item_height = None
             subwin_n_cols = win_n_cols - data['h_offset']
             start = current_row - subwin_n_rows + 1 if inverted else current_row
-            subwindow = window.derwin(
-                subwin_n_rows, subwin_n_cols, start, data['h_offset'])
-            attr = self._draw_item(subwindow, data, subwin_inverted)
-            self._subwindows.append((subwindow, attr))
+            subwindow = window.derwin(subwin_n_rows, subwin_n_cols, start, data['h_offset'])
+            self._subwindows.append((subwindow, data, subwin_inverted))
             available_rows -= (subwin_n_rows + 1)  # Add one for the blank line
             current_row += step * (subwin_n_rows + 1)
             if available_rows <= 0:
@@ -529,6 +499,23 @@ class Page(object):
             self.nav.flip((len(self._subwindows) - 1))
             return self._draw_content()
 
+        if self.nav.cursor_index >= len(self._subwindows):
+            # Don't allow the cursor to go over the number of subwindows
+            # This could happen if the window is resized and the cursor index is
+            # pushed out of bounds
+            self.nav.cursor_index = len(self._subwindows) - 1
+
+        # Now that the windows are setup, we can take a second pass through
+        # to draw the text onto each subwindow
+        for index, (win, data, inverted) in enumerate(self._subwindows):
+            if self.nav.absolute_index >= 0 and index == self.nav.cursor_index:
+                win.bkgd(str(' '), self.term.attr('Selected'))
+                with self.term.theme.turn_on_selected():
+                    self._draw_item(win, data, inverted)
+            else:
+                win.bkgd(str(' '), self.term.attr('Normal'))
+                self._draw_item(win, data, inverted)
+
         self._row += win_n_rows
 
     def _draw_footer(self):
@@ -536,51 +523,36 @@ class Page(object):
         n_rows, n_cols = self.term.stdscr.getmaxyx()
         window = self.term.stdscr.derwin(1, n_cols, self._row, 0)
         window.erase()
-        ch, attr = str(' '), curses.A_REVERSE | curses.A_BOLD | Color.CYAN
-        window.bkgd(ch, attr)
+        window.bkgd(str(' '), self.term.attr('HelpBar'))
 
         text = self.FOOTER.strip()
         self.term.add_line(window, text, 0, 0)
         self._row += 1
 
-    def _add_cursor(self):
-        self._edit_cursor(curses.A_REVERSE)
-
-    def _remove_cursor(self):
-        self._edit_cursor(curses.A_NORMAL)
-
     def _move_cursor(self, direction):
-        self._remove_cursor()
         # Note: ACS_VLINE doesn't like changing the attribute, so disregard the
         # redraw flag and opt to always redraw
         valid, redraw = self.nav.move(direction, len(self._subwindows))
         if not valid:
             self.term.flash()
-        self._add_cursor()
 
     def _move_page(self, direction):
-        self._remove_cursor()
         valid, redraw = self.nav.move_page(direction, len(self._subwindows)-1)
         if not valid:
             self.term.flash()
-        self._add_cursor()
 
-    def _edit_cursor(self, attribute):
+    def _prompt_period(self, order):
 
-        # Don't allow the cursor to go below page index 0
-        if self.nav.absolute_index < 0:
-            return
+        choices = {
+            '\n': order,
+            '1': '{0}-hour'.format(order),
+            '2': '{0}-day'.format(order),
+            '3': '{0}-week'.format(order),
+            '4': '{0}-month'.format(order),
+            '5': '{0}-year'.format(order),
+            '6': '{0}-all'.format(order)}
 
-        # Don't allow the cursor to go over the number of subwindows
-        # This could happen if the window is resized and the cursor index is
-        # pushed out of bounds
-        if self.nav.cursor_index >= len(self._subwindows):
-            self.nav.cursor_index = len(self._subwindows) - 1
-
-        window, attr = self._subwindows[self.nav.cursor_index]
-        if attr is not None:
-            attribute |= attr
-
-        n_rows, _ = window.getmaxyx()
-        for row in range(n_rows):
-            window.chgat(row, 0, 1, attribute)
+        message = docs.TIME_ORDER_MENU.strip().splitlines()
+        ch = self.term.show_notification(message)
+        ch = six.unichr(ch)
+        return choices.get(ch)
