@@ -3,12 +3,11 @@ from __future__ import unicode_literals
 
 import re
 import time
-import curses
 
 from . import docs
 from .content import SubredditContent
 from .page import Page, PageController, logged_in
-from .objects import Navigator, Color, Command
+from .objects import Navigator, Command
 from .submission_page import SubmissionPage
 from .subscription_page import SubscriptionPage
 from .exceptions import TemporaryFileError
@@ -35,9 +34,10 @@ class SubredditPage(Page):
         self.nav = Navigator(self.content.get)
         self.toggled_subreddit = None
 
-    @SubredditController.register(Command('REFRESH'))
     def refresh_content(self, order=None, name=None):
-        "Re-download all submissions and reset the page index"
+        """
+        Re-download all submissions and reset the page index
+        """
 
         order = order or self.content.order
 
@@ -60,9 +60,52 @@ class SubredditPage(Page):
         if not self.term.loader.exception:
             self.nav = Navigator(self.content.get)
 
+    @SubredditController.register(Command('SORT_HOT'))
+    def sort_content_hot(self):
+        if self.content.query:
+            self.refresh_content(order='relevance')
+        else:
+            self.refresh_content(order='hot')
+
+    @SubredditController.register(Command('SORT_TOP'))
+    def sort_content_top(self):
+        order = self._prompt_period('top')
+        if order is None:
+            self.term.show_notification('Invalid option')
+        else:
+            self.refresh_content(order=order)
+
+    @SubredditController.register(Command('SORT_RISING'))
+    def sort_content_rising(self):
+        if self.content.query:
+            order = self._prompt_period('comments')
+            if order is None:
+                self.term.show_notification('Invalid option')
+            else:
+                self.refresh_content(order=order)
+        else:
+            self.refresh_content(order='rising')
+
+    @SubredditController.register(Command('SORT_NEW'))
+    def sort_content_new(self):
+        self.refresh_content(order='new')
+
+    @SubredditController.register(Command('SORT_CONTROVERSIAL'))
+    def sort_content_controversial(self):
+        if self.content.query:
+            self.term.flash()
+        else:
+            order = self._prompt_period('controversial')
+            if order is None:
+                self.term.show_notification('Invalid option')
+            else:
+                self.refresh_content(order=order)
+
     @SubredditController.register(Command('SUBREDDIT_SEARCH'))
     def search_subreddit(self, name=None):
-        "Open a prompt to search the given subreddit"
+        """
+        Open a prompt to search the given subreddit
+        """
 
         name = name or self.content.name
 
@@ -78,7 +121,9 @@ class SubredditPage(Page):
 
     @SubredditController.register(Command('PROMPT'))
     def prompt_subreddit(self):
-        "Open a prompt to navigate to a different subreddit"
+        """
+        Open a prompt to navigate to a different subreddit"
+        """
 
         name = self.term.prompt_input('Enter page: /')
         if name is not None:
@@ -142,7 +187,9 @@ class SubredditPage(Page):
 
     @SubredditController.register(Command('SUBREDDIT_OPEN_IN_BROWSER'))
     def open_link(self):
-        "Open a link with the webbrowser"
+        """
+        Open a link with the webbrowser
+        """
 
         data = self.get_selected_item()
         if data['url_type'] == 'selfpost':
@@ -159,7 +206,9 @@ class SubredditPage(Page):
     @SubredditController.register(Command('SUBREDDIT_POST'))
     @logged_in
     def post_submission(self):
-        "Post a new submission to the given subreddit"
+        """
+        Post a new submission to the given subreddit
+        """
 
         # Check that the subreddit can be submitted to
         name = self.content.name
@@ -200,12 +249,14 @@ class SubredditPage(Page):
                 self.content = page.selected_subreddit
                 self.nav = Navigator(self.content.get)
             else:
-                self.refresh_content()
+                self.reload_page()
 
     @SubredditController.register(Command('SUBREDDIT_OPEN_SUBSCRIPTIONS'))
     @logged_in
     def open_subscriptions(self):
-        "Open user subscriptions page"
+        """
+        Open user subscriptions page
+        """
 
         with self.term.loader('Loading subscriptions'):
             page = SubscriptionPage(self.reddit, self.term, self.config,
@@ -224,7 +275,9 @@ class SubredditPage(Page):
     @SubredditController.register(Command('SUBREDDIT_OPEN_MULTIREDDITS'))
     @logged_in
     def open_multireddit_subscriptions(self):
-        "Open user multireddit subscriptions page"
+        """
+        Open user multireddit subscriptions page
+        """
 
         with self.term.loader('Loading multireddits'):
             page = SubscriptionPage(self.reddit, self.term, self.config,
@@ -250,51 +303,79 @@ class SubredditPage(Page):
         offset = 0 if not inverted else -(data['n_rows'] - n_rows)
 
         n_title = len(data['split_title'])
+        if data['url_full'] in self.config.history:
+            attr = self.term.attr('SubmissionTitleSeen')
+        else:
+            attr = self.term.attr('SubmissionTitle')
         for row, text in enumerate(data['split_title'], start=offset):
             if row in valid_rows:
-                self.term.add_line(win, text, row, 1, curses.A_BOLD)
+                self.term.add_line(win, text, row, 1, attr)
 
         row = n_title + offset
+        if data['url_full'] in self.config.history:
+            attr = self.term.attr('LinkSeen')
+        else:
+            attr = self.term.attr('Link')
         if row in valid_rows:
-            seen = (data['url_full'] in self.config.history)
-            link_color = Color.MAGENTA if seen else Color.BLUE
-            attr = curses.A_UNDERLINE | link_color
             self.term.add_line(win, '{url}'.format(**data), row, 1, attr)
 
         row = n_title + offset + 1
         if row in valid_rows:
-            self.term.add_line(win, '{score} '.format(**data), row, 1)
-            text, attr = self.term.get_arrow(data['likes'])
-            self.term.add_line(win, text, attr=attr)
-            self.term.add_line(win, ' {created} '.format(**data))
+
+            attr = self.term.attr('Score')
+            self.term.add_line(win, '{score}'.format(**data), row, 1, attr)
+            self.term.add_space(win)
+
+            arrow, attr = self.term.get_arrow(data['likes'])
+            self.term.add_line(win, arrow, attr=attr)
+            self.term.add_space(win)
+
+            attr = self.term.attr('Created')
+            self.term.add_line(win, '{created}'.format(**data), attr=attr)
 
             if data['comments'] is not None:
-                text, attr = '-', curses.A_BOLD
-                self.term.add_line(win, text, attr=attr)
-                self.term.add_line(win, ' {comments} '.format(**data))
+                attr = self.term.attr('Separator')
+                self.term.add_space(win)
+                self.term.add_line(win, '-', attr=attr)
+
+                attr = self.term.attr('CommentCount')
+                self.term.add_space(win)
+                self.term.add_line(win, '{comments}'.format(**data), attr=attr)
 
             if data['saved']:
-                text, attr = '[saved]', Color.GREEN
-                self.term.add_line(win, text, attr=attr)
+                attr = self.term.attr('Saved')
+                self.term.add_space(win)
+                self.term.add_line(win, '[saved]', attr=attr)
 
             if data['stickied']:
-                text, attr = '[stickied]', Color.GREEN
-                self.term.add_line(win, text, attr=attr)
+                attr = self.term.attr('Stickied')
+                self.term.add_space(win)
+                self.term.add_line(win, '[stickied]', attr=attr)
 
             if data['gold']:
-                text, attr = self.term.guilded
-                self.term.add_line(win, text, attr=attr)
+                attr = self.term.attr('Gold')
+                self.term.add_space(win)
+                self.term.add_line(win, self.term.guilded, attr=attr)
 
             if data['nsfw']:
-                text, attr = 'NSFW', (curses.A_BOLD | Color.RED)
-                self.term.add_line(win, text, attr=attr)
+                attr = self.term.attr('NSFW')
+                self.term.add_space(win)
+                self.term.add_line(win, 'NSFW', attr=attr)
 
         row = n_title + offset + 2
         if row in valid_rows:
-            text = '{author}'.format(**data)
-            self.term.add_line(win, text, row, 1, Color.GREEN)
-            text = ' /r/{subreddit}'.format(**data)
-            self.term.add_line(win, text, attr=Color.YELLOW)
+            attr = self.term.attr('SubmissionAuthor')
+            self.term.add_line(win, '{author}'.format(**data), row, 1, attr)
+            self.term.add_space(win)
+
+            attr = self.term.attr('SubmissionSubreddit')
+            self.term.add_line(win, '/r/{subreddit}'.format(**data), attr=attr)
+
             if data['flair']:
-                text = ' {flair}'.format(**data)
-                self.term.add_line(win, text, attr=Color.RED)
+                attr = self.term.attr('SubmissionFlair')
+                self.term.add_space(win)
+                self.term.add_line(win, '{flair}'.format(**data), attr=attr)
+
+        attr = self.term.attr('CursorBlock')
+        for y in range(n_rows):
+            self.term.addch(win, y, 0, str(' '), attr)
