@@ -80,8 +80,8 @@ def test_config_get_args():
     "Ensure that command line arguments are parsed properly"
 
     args = ['rtv',
+            'https://reddit.com/permalink •',
             '-s', 'cfb',
-            '-l', 'https://reddit.com/permalink •',
             '--log', 'logfile.log',
             '--config', 'configfile.cfg',
             '--ascii',
@@ -89,7 +89,9 @@ def test_config_get_args():
             '--non-persistent',
             '--clear-auth',
             '--copy-config',
-            '--enable-media']
+            '--enable-media',
+            '--theme', 'molokai',
+            '--list-themes']
 
     with mock.patch('sys.argv', ['rtv']):
         config_dict = Config.get_args()
@@ -111,6 +113,25 @@ def test_config_get_args():
         assert config['config'] == 'configfile.cfg'
         assert config['copy_config'] is True
         assert config['enable_media'] is True
+        assert config['theme'] == 'molokai'
+        assert config['list_themes'] is True
+
+
+def test_config_link_deprecated():
+
+    # Should still be able to specify the link using the old "-l"
+    args = ['rtv', '-l', 'https://reddit.com/option']
+    with mock.patch('sys.argv', args):
+        config_dict = Config.get_args()
+        config = Config(**config_dict)
+        assert config['link'] == 'https://reddit.com/option'
+
+    # But the positional argument should take preference
+    args = ['rtv', 'https://reddit.com/arg', '-l', 'https://reddit.com/option']
+    with mock.patch('sys.argv', args):
+        config_dict = Config.get_args()
+        config = Config(**config_dict)
+        assert config['link'] == 'https://reddit.com/arg'
 
 
 def test_config_from_file():
@@ -126,7 +147,8 @@ def test_config_from_file():
         'subreddit': 'cfb',
         'enable_media': True,
         'max_comment_cols': 150,
-        'hide_username': True}
+        'hide_username': True,
+        'theme': 'molokai'}
 
     bindings = {
         'REFRESH': 'r, <KEY_F5>',
