@@ -11,7 +11,8 @@ import pytest
 from rtv import exceptions
 from rtv.packages import praw
 from rtv.content import (
-    Content, SubmissionContent, SubredditContent, SubscriptionContent)
+    Content, SubmissionContent, SubredditContent, SubscriptionContent,
+    RequestHeaderRateLimiter)
 
 try:
     from unittest import mock
@@ -46,12 +47,12 @@ SUBREDDIT_PROMPTS = OrderedDict([
     ('user-1', ('/u/spez', '/u/spez', None)),
     ('user-new', ('/u/spez/new', '/u/spez', 'new')),
     ('user-top-all', ('/u/spez/top-all', '/u/spez', 'top-all')),
-    ('multi-0', ('/user/multi-mod/m/art', '/user/multi-mod/m/art', None)),
+    ('multi-0', ('/user/multi-mod/m/art', '/u/multi-mod/m/art', None)),
     ('multi-1', ('/u/multi-mod/m/art', '/u/multi-mod/m/art', None)),
     ('multi-top', ('/u/multi-mod/m/art/top', '/u/multi-mod/m/art', 'top')),
     ('multi-top-all', ('/u/multi-mod/m/art/top-all', '/u/multi-mod/m/art', 'top-all')),
     ('domain', ('/domain/python.org', '/domain/python.org', None)),
-    ('domain-top', ('/domain/python.org/top', '/domain/python.org', 'top')),
+    ('domain-new', ('/domain/python.org/new', '/domain/python.org', 'new')),
     ('domain-top-all', ('/domain/python.org/top-all', '/domain/python.org', 'top-all')),
 ])
 
@@ -62,6 +63,7 @@ SUBREDDIT_AUTH_PROMPTS = OrderedDict([
     ('me-top', ('/u/me/top', '/u/me', 'top')),
     ('me-top-all', ('/u/me/top-all', '/u/me', 'top-all')),
     ('user-saved', ('/u/saved', '/u/saved', None)),
+    ('me-multi', ('/u/me/m/redditpets/top-all', '/u/{username}/m/redditpets', 'top-all')),
 ])
 
 # All of these should raise an error when entered
@@ -396,6 +398,9 @@ def test_content_subreddit_from_name_authenticated(
     oauth.config.refresh_token = refresh_token
     oauth.authorize()
 
+    if '{username}' in name:
+        name = name.format(username=reddit.user.name)
+
     content = SubredditContent.from_name(reddit, prompt, terminal.loader)
     assert content.name == name
     assert content.order == order
@@ -411,6 +416,7 @@ def test_content_subreddit_from_name_invalid(prompt, reddit, terminal):
     # Must always have an argument because it gets displayed
     assert terminal.loader.exception.args[0]
 
+
 args, ids = SUBREDDIT_SEARCH_QUERIES.values(), list(SUBREDDIT_SEARCH_QUERIES)
 @pytest.mark.parametrize('prompt,query', args, ids=ids)
 def test_content_subreddit_from_name_query(prompt, query, reddit, terminal):
@@ -469,6 +475,41 @@ def test_content_subreddit_me(reddit, oauth, refresh_token, terminal):
         assert terminal.loader.exception.name == '/u/me'
 
 
+def test_content_subreddit_nsfw_filter(reddit, oauth, refresh_token, terminal):
+
+    # NSFW subreddits should load if not logged in
+    name = '/r/ImGoingToHellForThis'
+    SubredditContent.from_name(reddit, name, terminal.loader)
+
+    # Log in
+    oauth.config.refresh_token = refresh_token
+    oauth.authorize()
+
+    # Make sure the API parameter hasn't changed
+    assert reddit.user.over_18 is not None
+
+    # Turn on safe search
+    reddit.user.over_18 = False
+
+    # Should refuse to load this subreddit
+    with pytest.raises(exceptions.SubredditError):
+        name = '/r/ImGoingToHellForThis'
+        SubredditContent.from_name(reddit, name, terminal.loader)
+
+    # Should filter out all of the nsfw posts
+    name = '/r/ImGoingToHellForThis+python'
+    content = SubredditContent.from_name(reddit, name, terminal.loader)
+    for data in islice(content.iterate(0, 1), 50):
+        assert data['object'].over_18 is False
+
+    # Turn off safe search
+    reddit.user.over_18 = True
+
+    # The NSFW subreddit should load now
+    name = '/r/ImGoingToHellForThis'
+    SubredditContent.from_name(reddit, name, terminal.loader)
+
+
 def test_content_subscription(reddit, terminal):
 
     # Not logged in
@@ -520,3 +561,49 @@ def test_content_subscription_empty(reddit, terminal):
         with terminal.loader():
             SubscriptionContent.from_user(reddit, terminal.loader)
     assert isinstance(terminal.loader.exception, exceptions.SubscriptionError)
+
+
+def test_content_cache(reddit):
+
+    # Make sure the test suite is configured to use the custom handler
+    assert isinstance(reddit.handler, RequestHeaderRateLimiter)
+    assert not reddit.handler.cache
+
+    # A standard 200 response should be added to the cache
+    next(reddit.get_subreddit('python').get_hot())
+    request = list(reddit.handler.cache.values())[0]
+    assert request.url == 'https://api.reddit.com/r/python/.json'
+
+    # Clearing the cache should remove the request
+    reddit.handler.cache.clear()
+    assert not reddit.handler.cache
+
+    next(reddit.get_subreddit('python').get_hot())
+    assert reddit.handler.cache
+
+    # Evicting the cache should also remove the entry
+    reddit.handler.evict('https://api.reddit.com/r/python')
+    assert not reddit.handler.cache
+
+
+def test_content_rate_limit(reddit, oauth, refresh_token):
+
+    # Make sure the test suite is configured to use the custom handler
+    assert isinstance(reddit.handler, RequestHeaderRateLimiter)
+    assert not reddit.handler.cache
+
+    # unauthenticated requests don't return the x-ratelimit headers
+    # so they currently aren't limited
+    next(reddit.get_subreddit('python').get_hot())
+    assert reddit.handler.seconds_to_reset is None
+
+    oauth.config.refresh_token = refresh_token
+    oauth.authorize()
+
+    # But now that we're logged in the headers should be returned
+    next(reddit.get_subreddit('python').get_hot())
+    assert reddit.handler.seconds_to_reset
+
+    # Even though the headers were returned, the rate limiting should
+    # still not be triggering a delay for the next request
+    assert reddit.handler.next_request_timestamp is None
